"""
Mathematical Modeling Module

Mathematical modeling features using numpy, pandas, scipy, and scikit-learn.
"""

from .modeling import MathematicalModelingEngine

__all__ = ["MathematicalModelingEngine"]
