"""
Automated Feature Engineering Module

Provides classic and deep feature engineering capabilities:
- Mathematical transformations
- Aggregation features
- Interaction features
- Encoding strategies
- Time-series features
"""

from .engineer import FeatureEngineer

__all__ = ["FeatureEngineer"]
